@@ -1,8 +1,7 @@
 package Test::Smoke::SourceTree;
 use strict;
 
-use vars qw( $VERSION @EXPORT_OK %EXPORT_TAGS $NOCASE );
-$VERSION = '0.008';
+our $VERSION = '0.008';
 
 use File::Spec;
 use File::Find;
@@ -11,13 +10,13 @@ use Carp;
 use Test::Smoke::LogMixin;
 
 use base 'Exporter';
-%EXPORT_TAGS = (
+our %EXPORT_TAGS = (
     mani_const => [qw( &ST_MISSING &ST_UNDECLARED )],
     const      => [qw( &ST_MISSING &ST_UNDECLARED )],
 );
-@EXPORT_OK = @{ $EXPORT_TAGS{mani_const} };
+our @EXPORT_OK = @{ $EXPORT_TAGS{mani_const} };
 
-$NOCASE = $^O eq 'MSWin32' || $^O eq 'VMS';
+our $NOCASE = $^O eq 'MSWin32' || $^O eq 'VMS';
 
 =head1 NAME
 
@@ -222,7 +221,7 @@ sub check_MANIFEST {
     my %ignore = map {
         my $entry = $NOCASE ? uc $_ : $_;
         $entry => undef
-    } ( ".patch", "MANIFEST.SKIP", '.git', '.gitignore', @_ ),
+    } ( ".patch", "MANIFEST.SKIP", '.git', '.gitignore', '.mailmap', @_ ),
       keys %{ $self->_read_mani_file( 'MANIFEST.SKIP', 1 ) };
     $self->log_debug("Found %d entries in MANIFEST.SKIP", scalar(keys %ignore));
 
