@@ -11,8 +11,7 @@ use Test::Smoke::Syncer::Snapshot;
 use Test::Smoke::Syncer::FTP;
 use Test::Smoke::Syncer::Forest;
 
-use vars qw( $VERSION );
-$VERSION = '0.029';
+our $VERSION = '0.029';
 
 use Config;
 use Cwd qw( cwd abs_path);
@@ -106,7 +105,7 @@ my %CONFIG = (
 
 # synctype: git
     df_gitbin        => 'git',
-    df_gitorigin     => 'git://github.com/Perl/perl5.git',
+    df_gitorigin     => 'https://github.com/Perl/perl5.git',
     df_gitdir        => undef,
     df_gitdfbranch   => 'blead',
     df_gitbranchfile => undef,
