@@ -36,7 +36,7 @@ Do the actual syncing.
 
 There are 2 repositories, they both need to be updated:
 
-The first (proxy) repository has the perl5.git.perl.org repository as its
+The first (proxy) repository has the github.com/Perl repository as its
 (origin) remote. The second repository is used to run the smoker from.
 
 For the proxy-repository we do:
@@ -67,6 +67,7 @@ sub sync {
         my $cloneout = $gitbin->run(
             clone => $self->{gitorigin},
             $self->{gitdir},
+            ($^O eq 'MSWin32' ? ('--config', 'core.autocrlf=input') : ()),
             '2>&1'
         );
         if ( my $gitexit = $gitbin->exitcode ) {
@@ -97,12 +98,12 @@ sub sync {
         my $cloneout = $gitbin->run(
             clone         => $self->{gitdir},
             $self->{ddir},
+            ($^O eq 'MSWin32' ? ('--config', 'core.autocrlf=input') : ()),
             '2>&1'
         );
         if ( my $gitexit = $gitbin->exitcode ) {
             croak("Cannot make smoke clone: $self->{gitbin} exit $gitexit");
         }
-        $cloneout =~ s/%/%%/g;
         $self->log_debug("[git clone $self->{gitdir}]: $cloneout");
     }
 
@@ -152,7 +153,7 @@ sub get_git_branch {
     if (open my $fh, '<', $self->{gitbranchfile}) {
         $self->log_debug("Reading branch to smoke from: '$self->{gitbranchfile}'");
 
-        my $branch = <$fh>;
+        chomp( my $branch = <$fh> );
         close $fh;
         return $branch;
     }
